@@ -475,7 +475,7 @@ class FITSRecord(dict):
     def __init__(self, record):
         self.set_record(record)
 
-    def set_record(self, record, **kw):
+    def set_record(self, record, **keys):
         """
         check the record is valid and set keys in the dict
 
@@ -486,6 +486,13 @@ class FITSRecord(dict):
             card
         """
 
+        if not keys:
+            import warnings
+            warnings.warn(
+                "The keyword arguments '%s' are being ignored! This warning "
+                "will be an error in a future version of `fitsio`!",
+                DeprecationWarning)
+
         if isstring(record):
             card = FITSCard(record)
             self.update(card)
